@@ -32,6 +32,7 @@
 #endif
 
 #include <string.h>
+#include <gst/base/gstbitreader.h>
 #include <gst/base/gstbytereader.h>
 
 
@@ -122,21 +123,21 @@ static const guint8 mpeg4_zigzag_8x8[64] = {
 };
 
 static const VLCTable mpeg4_dmv_size_vlc_table[] = {
-  {0x00, 2, 0},
-  {0x02, 3, 1},
-  {0x03, 3, 2},
-  {0x04, 3, 3},
-  {0x05, 3, 4},
-  {0x06, 3, 5},
-  {0x0e, 4, 6},
-  {0x1e, 5, 7},
-  {0x3e, 6, 8},
-  {0x7e, 7, 9},
-  {0xfe, 8, 10},
-  {0x1fe, 9, 11},
-  {0x3fe, 10, 12},
-  {0x7fe, 11, 13},
-  {0xffe, 12, 14}
+  {0, 0x00, 2},
+  {1, 0x02, 3},
+  {2, 0x03, 3},
+  {3, 0x04, 3},
+  {4, 0x05, 3},
+  {5, 0x06, 3},
+  {6, 0x0e, 4},
+  {7, 0x1e, 5},
+  {8, 0x3e, 6},
+  {9, 0x7e, 7},
+  {10, 0xfe, 8},
+  {11, 0x1fe, 9},
+  {12, 0x3fe, 10},
+  {13, 0x7fe, 11},
+  {14, 0xffe, 12}
 };
 
 static void
@@ -346,7 +347,7 @@ compute_resync_marker_size (const GstMpeg4VideoObjectPlane * vop,
     }
   }
 
-  return off++;                 /* Take the following 1 into account */
+  return off;
 }
 
 /**
