@@ -218,19 +218,20 @@ context_create (GstVaapiContext * context)
       guint va_rate_control;
 
       /* Rate control */
-      attrib->type = VAConfigAttribRateControl;
-      if (!context_get_attribute (context, attrib->type, &value))
-        goto cleanup;
-
       va_rate_control = from_GstVaapiRateControl (config->rc_mode);
-      if ((value & va_rate_control) != va_rate_control) {
-        GST_ERROR ("unsupported %s rate control",
-            string_of_VARateControl (va_rate_control));
-        goto cleanup;
-      }
-      attrib->value = va_rate_control;
-      attrib++;
+      if (va_rate_control != VA_RC_NONE) {
+        attrib->type = VAConfigAttribRateControl;
+        if (!context_get_attribute (context, attrib->type, &value))
+          goto cleanup;
 
+        if ((value & va_rate_control) != va_rate_control) {
+          GST_ERROR ("unsupported %s rate control",
+              string_of_VARateControl (va_rate_control));
+          goto cleanup;
+        }
+        attrib->value = va_rate_control;
+        attrib++;
+      }
       /* Packed headers */
       if (config->packed_headers) {
         attrib->type = VAConfigAttribEncPackedHeaders;
@@ -245,6 +246,23 @@ context_create (GstVaapiContext * context)
         attrib->value = config->packed_headers;
         attrib++;
       }
+#if VA_CHECK_VERSION(0,37,0)
+      if (cip->profile == GST_VAAPI_PROFILE_JPEG_BASELINE) {
+        VAConfigAttribValEncJPEG jpeg_attrib_val;
+        /* JPEG Encoding Attribute */
+        attrib->type = VAConfigAttribEncJPEG;
+        if (!context_get_attribute (context, attrib->type, &value))
+          goto cleanup;
+        jpeg_attrib_val.value = attrib->value;
+        /* Set JPEG profile attribs */
+        jpeg_attrib_val.bits.arithmatic_coding_mode = 0;
+        jpeg_attrib_val.bits.progressive_dct_mode = 0;
+        jpeg_attrib_val.bits.non_interleaved_mode = 1;
+        jpeg_attrib_val.bits.differential_mode = 0;
+        attrib->value = jpeg_attrib_val.value;
+        attrib++;
+      }
+#endif
       break;
     }
 #endif
