@@ -128,11 +128,16 @@ pts_eval(PTSGenerator *tsg, GstClockTime pic_pts, guint pic_tsn)
     GstClockTime pts;
 
     if (!GST_CLOCK_TIME_IS_VALID(tsg->gop_pts))
-        tsg->gop_pts = 0;
+        tsg->gop_pts = pts_get_duration(tsg, pic_tsn);
 
     pts = pic_pts;
     if (!GST_CLOCK_TIME_IS_VALID (pts))
        pts = tsg->gop_pts + pts_get_duration(tsg, tsg->ovl_tsn * 1024 + pic_tsn);
+    else if (pts == tsg->gop_pts) {
+        /* The picture following the GOP header shall be an I-frame.
+           So we can compensate for the GOP start time from here */
+        tsg->gop_pts -= pts_get_duration(tsg, pic_tsn);
+    }
 
     if (!GST_CLOCK_TIME_IS_VALID(tsg->max_pts) || tsg->max_pts < pts)
         tsg->max_pts = pts;
@@ -308,6 +313,8 @@ gst_vaapi_decoder_mpeg2_close(GstVaapiDecoderMpeg2 *decoder)
     gst_vaapi_parser_info_mpeg2_replace(&priv->quant_matrix, NULL);
     gst_vaapi_parser_info_mpeg2_replace(&priv->slice_hdr, NULL);
 
+    priv->state = 0;
+
     gst_vaapi_dpb_replace(&priv->dpb, NULL);
 }
 
@@ -550,7 +557,7 @@ error:
 
 drop_frame:
     priv->state &= GST_MPEG_VIDEO_STATE_VALID_SEQ_HEADERS;
-    return GST_VAAPI_DECODER_STATUS_DROP_FRAME;
+    return (GstVaapiDecoderStatus) GST_VAAPI_DECODER_STATUS_DROP_FRAME;
 }
 
 static GstVaapiDecoderStatus
@@ -774,7 +781,8 @@ decode_sequence_end(GstVaapiDecoderMpeg2 *decoder)
 {
     GstVaapiDecoderMpeg2Private * const priv = &decoder->priv;
 
-    gst_vaapi_dpb_flush(priv->dpb);
+    if (priv->dpb)
+        gst_vaapi_dpb_flush(priv->dpb);
     return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
 
@@ -1103,6 +1111,9 @@ parse_slice(GstVaapiDecoderMpeg2 *decoder,
                     GST_MPEG_VIDEO_STATE_GOT_PIC_HDR|
                     GST_MPEG_VIDEO_STATE_GOT_PIC_EXT);
 
+    if (!is_valid_state(decoder, GST_MPEG_VIDEO_STATE_VALID_PIC_HEADERS))
+        return GST_VAAPI_DECODER_STATUS_SUCCESS;
+
     if (!gst_vaapi_parser_info_mpeg2_ensure(&priv->slice_hdr)) {
         GST_ERROR("failed to allocate parser info for slice header");
         return GST_VAAPI_DECODER_STATUS_ERROR_ALLOCATION_FAILED;
@@ -1135,8 +1146,6 @@ decode_slice(GstVaapiDecoderMpeg2 *decoder, GstVaapiDecoderUnit *unit)
         GST_VAAPI_DECODER_CODEC_FRAME(decoder)->input_buffer;
     GstMapInfo map_info;
 
-    GST_DEBUG("slice %d (%u bytes)", slice_hdr->mb_row, unit->size);
-
     if (!is_valid_state(decoder, GST_MPEG_VIDEO_STATE_VALID_PIC_HEADERS))
         return GST_VAAPI_DECODER_STATUS_SUCCESS;
 
@@ -1145,6 +1154,8 @@ decode_slice(GstVaapiDecoderMpeg2 *decoder, GstVaapiDecoderUnit *unit)
         return GST_VAAPI_DECODER_STATUS_ERROR_UNKNOWN;
     }
 
+    GST_DEBUG("slice %d (%u bytes)", slice_hdr->mb_row, unit->size);
+
     slice = GST_VAAPI_SLICE_NEW(MPEG2, decoder,
         (map_info.data + unit->offset), unit->size);
     gst_buffer_unmap(buffer, &map_info);
@@ -1537,7 +1548,8 @@ gst_vaapi_decoder_mpeg2_flush(GstVaapiDecoder *base_decoder)
         GST_VAAPI_DECODER_MPEG2_CAST(base_decoder);
     GstVaapiDecoderMpeg2Private * const priv = &decoder->priv;
 
-    gst_vaapi_dpb_flush(priv->dpb);
+    if (priv->dpb)
+      gst_vaapi_dpb_flush(priv->dpb);
     return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
 
