@@ -63,6 +63,7 @@ struct _GstVaapiDecoderVC1Private {
     GstVaapiDpb                *dpb;
     gint32                      next_poc;
     guint8                     *rbdu_buffer;
+    guint8                      rndctrl;
     guint                       rbdu_buffer_size;
     guint                       is_opened               : 1;
     guint                       is_first_field          : 1;
@@ -173,6 +174,7 @@ gst_vaapi_decoder_vc1_create(GstVaapiDecoder *base_decoder)
     GstVaapiDecoderVC1Private * const priv = &decoder->priv;
 
     priv->profile = (GstVaapiProfile)0;
+    priv->rndctrl = 0;
     return TRUE;
 }
 
@@ -668,7 +670,6 @@ fill_picture_structc(GstVaapiDecoderVC1 *decoder, GstVaapiPicture *picture)
     pic_param->cbp_table                                            = pic->cbptab;
     pic_param->mb_mode_table                                        = 0; /* XXX: interlaced frame */
     pic_param->range_reduction_frame                                = pic->rangeredfrm;
-    pic_param->rounding_control                                     = 0; /* advanced profile only */
     pic_param->post_processing                                      = 0; /* advanced profile only */
     pic_param->picture_resolution_index                             = pic->respic;
     pic_param->luma_scale                                           = pic->lumscale;
@@ -686,6 +687,16 @@ fill_picture_structc(GstVaapiDecoderVC1 *decoder, GstVaapiPicture *picture)
     pic_param->transform_fields.bits.mb_level_transform_type_flag   = pic->ttmbf;
     pic_param->transform_fields.bits.frame_level_transform_type     = pic->ttfrm;
     pic_param->transform_fields.bits.transform_ac_codingset_idx2    = pic->transacfrm2;
+
+    /* Refer to 8.3.7 Rounding control for Simple and Main Profile  */
+    if (frame_hdr->ptype == GST_VC1_PICTURE_TYPE_I ||
+        frame_hdr->ptype == GST_VC1_PICTURE_TYPE_BI)
+        priv->rndctrl = 1;
+    else if (frame_hdr->ptype == GST_VC1_PICTURE_TYPE_P)
+        priv->rndctrl ^= 1;
+
+    pic_param->rounding_control = priv->rndctrl;
+
     return TRUE;
 }
 
@@ -1060,6 +1071,13 @@ decode_ebdu(GstVaapiDecoderVC1 *decoder, GstVC1BDU *ebdu)
     case GST_VC1_END_OF_SEQ:
         status = decode_sequence_end(decoder);
         break;
+    case GST_VC1_FIELD_USER:
+    case GST_VC1_FRAME_USER:
+    case GST_VC1_ENTRY_POINT_USER:
+    case GST_VC1_SEQUENCE_USER:
+        /* Let's just ignore them */
+        status = GST_VAAPI_DECODER_STATUS_SUCCESS;
+        break;
     default:
         GST_WARNING("unsupported BDU type %d", ebdu->type);
         status = GST_VAAPI_DECODER_STATUS_ERROR_BITSTREAM_PARSER;
@@ -1106,6 +1124,7 @@ gst_vaapi_decoder_vc1_decode_codec_data(GstVaapiDecoder *base_decoder,
     gint width, height;
     guint32 format;
     gint version;
+    const gchar *s;
 
     priv->has_codec_data = TRUE;
 
@@ -1118,7 +1137,10 @@ gst_vaapi_decoder_vc1_decode_codec_data(GstVaapiDecoder *base_decoder,
 
     caps = GST_VAAPI_DECODER_CODEC_STATE(decoder)->caps;
     structure = gst_caps_get_structure(caps, 0);
-    if (!gst_structure_get_fourcc(structure, "format", &format)) {
+    s = gst_structure_get_string(structure, "format");
+    if (s && strlen(s) == 4) {
+        format = GST_MAKE_FOURCC(s[0], s[1], s[2], s[3]);
+    } else {
         /* Try to determine format from "wmvversion" property */
         if (gst_structure_get_int(structure, "wmvversion", &version))
             format = (version >= 1 && version <= 3) ?
