@@ -163,6 +163,10 @@ gst_vaapi_display_type_get_type (void)
     {GST_VAAPI_DISPLAY_TYPE_GLX,
         "VA/GLX display", "glx"},
 #endif
+#if USE_EGL
+    {GST_VAAPI_DISPLAY_TYPE_EGL,
+        "VA/EGL display", "egl"},
+#endif
 #if USE_WAYLAND
     {GST_VAAPI_DISPLAY_TYPE_WAYLAND,
         "VA/Wayland display", "wayland"},
@@ -201,6 +205,10 @@ gst_vaapi_display_type_is_compatible (GstVaapiDisplayType type1,
       if (type2 == GST_VAAPI_DISPLAY_TYPE_X11)
         return TRUE;
       break;
+    case GST_VAAPI_DISPLAY_TYPE_X11:
+      if (type2 == GST_VAAPI_DISPLAY_TYPE_EGL)
+        return TRUE;
+      break;
     default:
       break;
   }
@@ -557,6 +565,7 @@ ensure_profiles (GstVaapiDisplay * display)
           g_array_append_val (priv->decoders, config);
           break;
         case GST_VAAPI_ENTRYPOINT_SLICE_ENCODE:
+        case GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE:
           g_array_append_val (priv->encoders, config);
           break;
       }
@@ -2133,8 +2142,9 @@ gst_vaapi_display_has_opengl (GstVaapiDisplay * display)
 {
   GstVaapiDisplayClass *klass;
 
-  g_return_val_if_fail (display != NULL, NULL);
+  g_return_val_if_fail (display != NULL, FALSE);
 
   klass = GST_VAAPI_DISPLAY_GET_CLASS (display);
-  return (klass->display_type == GST_VAAPI_DISPLAY_TYPE_GLX);
+  return (klass->display_type == GST_VAAPI_DISPLAY_TYPE_GLX ||
+          klass->display_type == GST_VAAPI_DISPLAY_TYPE_EGL);
 }
