@@ -512,14 +512,22 @@ get_config_attribute (GstVaapiEncoder * encoder, VAConfigAttribType type,
 {
   GstVaapiProfile profile;
   VAProfile va_profile;
+  VAEntrypoint va_entrypoint;
+  const GstVaapiEncoderClassData *const cdata =
+      GST_VAAPI_ENCODER_GET_CLASS (encoder)->class_data;
 
   profile = get_profile (encoder);
   if (!profile)
     return FALSE;
-
   va_profile = gst_vaapi_profile_get_va_profile (profile);
+
+  if (cdata->codec != GST_VAAPI_CODEC_JPEG)
+    va_entrypoint = VAEntrypointEncSlice;
+  else
+    va_entrypoint = VAEntrypointEncPicture;
+
   return gst_vaapi_get_config_attribute (encoder->display, va_profile,
-      VAEntrypointEncSlice, type, out_value_ptr);
+      va_entrypoint, type, out_value_ptr);
 }
 
 /* Determines the set of supported packed headers */
@@ -539,6 +547,15 @@ get_packed_headers (GstVaapiEncoder * encoder)
 
   encoder->got_packed_headers = TRUE;
   encoder->packed_headers = cdata->packed_headers & value;
+
+  if (cdata->codec == GST_VAAPI_CODEC_JPEG) {
+#if !VA_CHECK_VERSION(0,37,1)
+    encoder->packed_headers = VA_ENC_PACKED_HEADER_RAW_DATA;
+    GST_DEBUG ("Hard coding the packed header flag value to VA_ENC_PACKED_HEADER_RAW_DATA,"
+        "This is a work around for the driver bug");
+#endif
+  }
+
   return encoder->packed_headers;
 }
 
@@ -550,10 +567,16 @@ set_context_info (GstVaapiEncoder * encoder)
   GstVaapiConfigInfoEncoder *const config = &cip->config.encoder;
   const GstVideoFormat format =
     GST_VIDEO_INFO_FORMAT (GST_VAAPI_ENCODER_VIDEO_INFO (encoder));
+  const GstVaapiEncoderClassData *const cdata =
+      GST_VAAPI_ENCODER_GET_CLASS (encoder)->class_data;
 
   cip->usage = GST_VAAPI_CONTEXT_USAGE_ENCODE;
   cip->profile = encoder->profile;
   cip->entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE;
+  if (cdata->codec != GST_VAAPI_CODEC_JPEG)
+    cip->entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE;
+  else
+    cip->entrypoint = GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE;
   cip->chroma_type = gst_vaapi_video_format_get_chroma_type (format);
   cip->width = GST_VAAPI_ENCODER_WIDTH (encoder);
   cip->height = GST_VAAPI_ENCODER_HEIGHT (encoder);
@@ -562,6 +585,13 @@ set_context_info (GstVaapiEncoder * encoder)
   if (!cip->chroma_type  && (format != GST_VIDEO_FORMAT_ENCODED))
     goto error_unsupported_format;
 
+  if (cip->chroma_type != GST_VAAPI_CHROMA_TYPE_YUV420 &&
+      format != GST_VIDEO_FORMAT_ENCODED) {
+    GST_ERROR ("We are only supporting YUV:4:2:0 for encoding,"
+        "please try to use vaapipostproc to convert the input format!");
+    goto error_unsupported_format;
+  }
+  
   memset (config, 0, sizeof (*config));
   config->rc_mode = GST_VAAPI_ENCODER_RATE_CONTROL (encoder);
   config->packed_headers = get_packed_headers (encoder);
@@ -840,7 +870,7 @@ gst_vaapi_encoder_set_rate_control (GstVaapiEncoder * encoder,
     goto error_operation_failed;
 
   rate_control_mask = get_rate_control_mask (encoder);
-  if (!(rate_control_mask & (1U << rate_control)))
+  if (rate_control_mask && !(rate_control_mask & (1U << rate_control)))
     goto error_unsupported_rate_control;
 
   encoder->rate_control = rate_control;
