@@ -34,7 +34,7 @@ G_BEGIN_DECLS
  * GstVaapiSurfaceProxyFlags:
  * @GST_VAAPI_SURFACE_PROXY_FLAG_INTERLACED: interlaced frame
  * @GST_VAAPI_SURFACE_PROXY_FLAG_TFF: top-field-first
- * @GST_VAAPI_SURFACE_PROXY_FLAG_RFF: repeat-field-first
+ * @GST_VAAPI_SURFACE_PROXY_FLAG_RFF: repeat-first-field
  * @GST_VAAPI_SURFACE_PROXY_FLAG_ONEFIELD: only one field is available
  * @GST_VAAPI_SURFACE_PROXY_FLAG_FFB: first frame in bundle, e.g. the first
  *   view component of a MultiView Coded (MVC) frame
@@ -42,13 +42,14 @@ G_BEGIN_DECLS
  *
  * Flags for #GstVaapiDecoderFrame.
  */
-typedef enum {
-    GST_VAAPI_SURFACE_PROXY_FLAG_INTERLACED     = (1 << 0),
-    GST_VAAPI_SURFACE_PROXY_FLAG_TFF            = (1 << 1),
-    GST_VAAPI_SURFACE_PROXY_FLAG_RFF            = (1 << 2),
-    GST_VAAPI_SURFACE_PROXY_FLAG_ONEFIELD       = (1 << 3),
-    GST_VAAPI_SURFACE_PROXY_FLAG_FFB            = (1 << 4),
-    GST_VAAPI_SURFACE_PROXY_FLAG_LAST           = (1 << 8)
+typedef enum
+{
+  GST_VAAPI_SURFACE_PROXY_FLAG_INTERLACED = (1 << 0),
+  GST_VAAPI_SURFACE_PROXY_FLAG_TFF = (1 << 1),
+  GST_VAAPI_SURFACE_PROXY_FLAG_RFF = (1 << 2),
+  GST_VAAPI_SURFACE_PROXY_FLAG_ONEFIELD = (1 << 3),
+  GST_VAAPI_SURFACE_PROXY_FLAG_FFB = (1 << 4),
+  GST_VAAPI_SURFACE_PROXY_FLAG_LAST = (1 << 8)
 } GstVaapiSurfaceProxyFlags;
 
 /**
@@ -58,7 +59,7 @@ typedef enum {
  * Macro that evaluates to the #GstVaapiSurface of @proxy.
  */
 #define GST_VAAPI_SURFACE_PROXY_SURFACE(proxy) \
-    gst_vaapi_surface_proxy_get_surface(proxy)
+  gst_vaapi_surface_proxy_get_surface (proxy)
 
 /**
  * GST_VAAPI_SURFACE_PROXY_SURFACE_ID:
@@ -68,7 +69,7 @@ typedef enum {
  * surface.
  */
 #define GST_VAAPI_SURFACE_PROXY_SURFACE_ID(proxy) \
-    gst_vaapi_surface_proxy_get_surface_id(proxy)
+  gst_vaapi_surface_proxy_get_surface_id (proxy)
 
 /**
  * GST_VAAPI_SURFACE_PROXY_VIEW_ID:
@@ -78,7 +79,7 @@ typedef enum {
  * surface.
  */
 #define GST_VAAPI_SURFACE_PROXY_VIEW_ID(proxy) \
-    gst_vaapi_surface_proxy_get_view_id(proxy)
+  gst_vaapi_surface_proxy_get_view_id (proxy)
 
 /**
  * GST_VAAPI_SURFACE_PROXY_TIMESTAMP:
@@ -88,7 +89,7 @@ typedef enum {
  * underlying @proxy surface.
  */
 #define GST_VAAPI_SURFACE_PROXY_TIMESTAMP(proxy) \
-    gst_vaapi_surface_proxy_get_timestamp(proxy)
+  gst_vaapi_surface_proxy_get_timestamp (proxy)
 
 /**
  * GST_VAAPI_SURFACE_PROXY_DURATION:
@@ -98,52 +99,55 @@ typedef enum {
  * underlying @proxy surface.
  */
 #define GST_VAAPI_SURFACE_PROXY_DURATION(proxy) \
-    gst_vaapi_surface_proxy_get_duration(proxy)
+  gst_vaapi_surface_proxy_get_duration (proxy)
 
 GstVaapiSurfaceProxy *
-gst_vaapi_surface_proxy_new_from_pool(GstVaapiSurfacePool *pool);
+gst_vaapi_surface_proxy_new (GstVaapiSurface * surface);
 
 GstVaapiSurfaceProxy *
-gst_vaapi_surface_proxy_copy(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_new_from_pool (GstVaapiSurfacePool * pool);
 
 GstVaapiSurfaceProxy *
-gst_vaapi_surface_proxy_ref(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_copy (GstVaapiSurfaceProxy * proxy);
+
+GstVaapiSurfaceProxy *
+gst_vaapi_surface_proxy_ref (GstVaapiSurfaceProxy * proxy);
 
 void
-gst_vaapi_surface_proxy_unref(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_unref (GstVaapiSurfaceProxy * proxy);
 
 void
-gst_vaapi_surface_proxy_replace(GstVaapiSurfaceProxy **old_proxy_ptr,
-    GstVaapiSurfaceProxy *new_proxy);
+gst_vaapi_surface_proxy_replace (GstVaapiSurfaceProxy ** old_proxy_ptr,
+    GstVaapiSurfaceProxy * new_proxy);
 
 guint
-gst_vaapi_surface_proxy_get_flags(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_flags (GstVaapiSurfaceProxy * proxy);
 
 GstVaapiSurface *
-gst_vaapi_surface_proxy_get_surface(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_surface (GstVaapiSurfaceProxy * proxy);
 
 GstVaapiID
-gst_vaapi_surface_proxy_get_surface_id(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_surface_id (GstVaapiSurfaceProxy * proxy);
 
 guintptr
-gst_vaapi_surface_proxy_get_view_id(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_view_id (GstVaapiSurfaceProxy * proxy);
 
 GstClockTime
-gst_vaapi_surface_proxy_get_timestamp(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_timestamp (GstVaapiSurfaceProxy * proxy);
 
 GstClockTime
-gst_vaapi_surface_proxy_get_duration(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_duration (GstVaapiSurfaceProxy * proxy);
 
 void
-gst_vaapi_surface_proxy_set_destroy_notify(GstVaapiSurfaceProxy *proxy,
+gst_vaapi_surface_proxy_set_destroy_notify (GstVaapiSurfaceProxy * proxy,
     GDestroyNotify destroy_func, gpointer user_data);
 
 const GstVaapiRectangle *
-gst_vaapi_surface_proxy_get_crop_rect(GstVaapiSurfaceProxy *proxy);
+gst_vaapi_surface_proxy_get_crop_rect (GstVaapiSurfaceProxy * proxy);
 
 void
-gst_vaapi_surface_proxy_set_crop_rect(GstVaapiSurfaceProxy *proxy,
-    const GstVaapiRectangle *crop_rect);
+gst_vaapi_surface_proxy_set_crop_rect (GstVaapiSurfaceProxy * proxy,
+    const GstVaapiRectangle * crop_rect);
 
 G_END_DECLS
 
