@@ -128,7 +128,7 @@ _gst_vaapi_window_glx_create_context (GstVaapiWindow * window,
 {
   GstVaapiWindowGLXPrivate *const priv =
       GST_VAAPI_WINDOW_GLX_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_XDISPLAY (window);
+  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
   GLContextState parent_cs;
 
   parent_cs.display = dpy;
@@ -136,8 +136,7 @@ _gst_vaapi_window_glx_create_context (GstVaapiWindow * window,
   parent_cs.context = foreign_context;
 
   GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
-  priv->gl_context = gl_create_context (dpy,
-      GST_VAAPI_OBJECT_XSCREEN (window), &parent_cs);
+  priv->gl_context = gl_create_context (dpy, DefaultScreen (dpy), &parent_cs);
   if (!priv->gl_context) {
     GST_DEBUG ("could not create GLX context");
     goto end;
@@ -207,15 +206,15 @@ gst_vaapi_window_glx_ensure_context (GstVaapiWindow * window,
   return TRUE;
 }
 
-static Visual *
-gst_vaapi_window_glx_get_visual (GstVaapiWindow * window)
+static guintptr
+gst_vaapi_window_glx_get_visual_id (GstVaapiWindow * window)
 {
   GstVaapiWindowGLXPrivate *const priv =
       GST_VAAPI_WINDOW_GLX_GET_PRIVATE (window);
 
   if (!_gst_vaapi_window_glx_ensure_context (window, NULL))
-    return NULL;
-  return priv->gl_context->visual->visual;
+    return 0;
+  return priv->gl_context->visual->visualid;
 }
 
 static void
@@ -223,7 +222,7 @@ gst_vaapi_window_glx_destroy_colormap (GstVaapiWindow * window)
 {
   GstVaapiWindowGLXPrivate *const priv =
       GST_VAAPI_WINDOW_GLX_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_XDISPLAY (window);
+  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
 
   if (priv->cmap) {
     if (!window->use_foreign_window) {
@@ -240,8 +239,7 @@ gst_vaapi_window_glx_create_colormap (GstVaapiWindow * window)
 {
   GstVaapiWindowGLXPrivate *const priv =
       GST_VAAPI_WINDOW_GLX_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_XDISPLAY (window);
-  int screen;
+  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
   XWindowAttributes wattr;
   gboolean success = FALSE;
 
@@ -252,9 +250,7 @@ gst_vaapi_window_glx_create_colormap (GstVaapiWindow * window)
       GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
       x11_trap_errors ();
       /* XXX: add a GstVaapiDisplayX11:x11-screen property? */
-      screen = GST_VAAPI_OBJECT_XSCREEN (window);
-      priv->cmap = XCreateColormap (dpy,
-          RootWindow (dpy, screen),
+      priv->cmap = XCreateColormap (dpy, RootWindow (dpy, DefaultScreen (dpy)),
           priv->gl_context->visual->visual, AllocNone);
       success = x11_untrap_errors () == 0;
       GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
@@ -272,10 +268,10 @@ gst_vaapi_window_glx_create_colormap (GstVaapiWindow * window)
   return priv->cmap;
 }
 
-static Colormap
+static guintptr
 gst_vaapi_window_glx_get_colormap (GstVaapiWindow * window)
 {
-  return gst_vaapi_window_glx_create_colormap (window);
+  return GPOINTER_TO_SIZE (gst_vaapi_window_glx_create_colormap (window));
 }
 
 static gboolean
@@ -285,7 +281,7 @@ gst_vaapi_window_glx_resize (GstVaapiWindow * window, guint width, guint height)
       GST_VAAPI_WINDOW_GLX_GET_PRIVATE (window);
   const GstVaapiWindowGLXClass *const klass =
       GST_VAAPI_WINDOW_GLX_GET_CLASS (window);
-  Display *const dpy = GST_VAAPI_OBJECT_XDISPLAY (window);
+  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
   GLContextState old_cs;
 
   if (!klass->parent_resize (window, width, height))
@@ -324,8 +320,8 @@ gst_vaapi_window_glx_class_init (GstVaapiWindowGLXClass * klass)
   klass->parent_resize = window_class->resize;
   klass->parent_finalize = GST_VAAPI_OBJECT_CLASS (klass)->finalize;
   window_class->resize = gst_vaapi_window_glx_resize;
-  xwindow_class->get_visual = gst_vaapi_window_glx_get_visual;
-  xwindow_class->get_colormap = gst_vaapi_window_glx_get_colormap;
+  window_class->get_visual_id = gst_vaapi_window_glx_get_visual_id;
+  window_class->get_colormap = gst_vaapi_window_glx_get_colormap;
 }
 
 GST_VAAPI_OBJECT_DEFINE_CLASS_WITH_CODE (GstVaapiWindowGLX,
@@ -351,8 +347,9 @@ gst_vaapi_window_glx_new (GstVaapiDisplay * display, guint width, guint height)
   g_return_val_if_fail (GST_VAAPI_IS_DISPLAY_GLX (display), NULL);
 
   window =
-      gst_vaapi_window_new (GST_VAAPI_WINDOW_CLASS (gst_vaapi_window_glx_class
-          ()), display, width, height);
+      gst_vaapi_window_new_internal (GST_VAAPI_WINDOW_CLASS
+      (gst_vaapi_window_glx_class ()), display, GST_VAAPI_ID_INVALID, width,
+      height);
   if (!window)
     return NULL;
 
@@ -388,8 +385,8 @@ gst_vaapi_window_glx_new_with_xid (GstVaapiDisplay * display, Window xid)
   g_return_val_if_fail (xid != None, NULL);
 
   window =
-      gst_vaapi_window_new_from_native (GST_VAAPI_WINDOW_CLASS
-      (gst_vaapi_window_glx_class ()), display, GINT_TO_POINTER (xid));
+      gst_vaapi_window_new_internal (GST_VAAPI_WINDOW_CLASS
+      (gst_vaapi_window_glx_class ()), display, xid, 0, 0);
   if (!window)
     return NULL;
 
