@@ -140,6 +140,22 @@ gst_vaapi_decoder_state_changed (GstVaapiDecoder * decoder,
     return;
 }
 
+static GstVideoCodecState *
+copy_video_codec_state (const GstVideoCodecState * in_state)
+{
+  GstVideoCodecState *state;
+
+  g_return_val_if_fail (in_state != NULL, NULL);
+
+  state = g_slice_new0 (GstVideoCodecState);
+  state->ref_count = 1;
+  state->info = in_state->info;
+  state->caps = gst_caps_copy (in_state->caps);
+  state->codec_data = gst_buffer_copy_deep (in_state->codec_data);
+
+  return state;
+}
+
 static gboolean
 gst_vaapi_decode_input_state_replace (GstVaapiDecode * decode,
     const GstVideoCodecState * new_state)
@@ -147,15 +163,19 @@ gst_vaapi_decode_input_state_replace (GstVaapiDecode * decode,
   if (decode->input_state) {
     if (new_state) {
       const GstCaps *curcaps = decode->input_state->caps;
-      if (gst_caps_is_always_compatible (curcaps, new_state->caps))
+      /* If existing caps are equal of the new state, keep the
+       * existing state without renegotiating. */
+      if (gst_caps_is_strictly_equal (curcaps, new_state->caps)) {
+        GST_DEBUG ("Ignoring new caps %" GST_PTR_FORMAT
+            " since are equal to current ones", new_state->caps);
         return FALSE;
+      }
     }
     gst_video_codec_state_unref (decode->input_state);
   }
 
   if (new_state)
-    decode->input_state = gst_video_codec_state_ref
-        ((GstVideoCodecState *) new_state);
+    decode->input_state = copy_video_codec_state (new_state);
   else
     decode->input_state = NULL;
 
@@ -232,8 +252,8 @@ gst_vaapidecode_update_src_caps (GstVaapiDecode * decode)
   gint fps_d = GST_VIDEO_INFO_FPS_D (vi);
   if (fps_n <= 0 || fps_d <= 0) {
     GST_DEBUG_OBJECT (decode, "forcing 25/1 framerate for latency calculation");
-    fps_n = 1;
-    fps_d = 25;
+    fps_n = 25;
+    fps_d = 1;
   }
 
   /* For parsing/preparation purposes we'd need at least 1 frame
@@ -698,8 +718,6 @@ gst_vaapidecode_reset_full (GstVaapiDecode * decode, GstCaps * caps,
 {
   GstVaapiCodec codec;
 
-  decode->has_texture_upload_meta = FALSE;
-
   /* Reset tracked frame size */
   decode->current_frame_size = 0;
 
