@@ -32,6 +32,10 @@
 #include <gst/vaapi/gstvaapidisplay.h>
 #include "gstvaapiuploader.h"
 
+#ifdef HAVE_GST_GL_GL_H
+# include <gst/gl/gstglcontext.h>
+#endif
+
 G_BEGIN_DECLS
 
 typedef struct _GstVaapiPluginBase GstVaapiPluginBase;
@@ -126,25 +130,23 @@ struct _GstVaapiPluginBase
   gboolean sinkpad_caps_is_raw;
   GstVideoInfo sinkpad_info;
   GstPadQueryFunction sinkpad_query;
-#if GST_CHECK_VERSION(1,0,0)
   GstBufferPool *sinkpad_buffer_pool;
   guint sinkpad_buffer_size;
-#endif
 
   GstPad *srcpad;
   GstCaps *srcpad_caps;
   gboolean srcpad_caps_changed;
   GstVideoInfo srcpad_info;
   GstPadQueryFunction srcpad_query;
-#if GST_CHECK_VERSION(1,0,0)
   GstBufferPool *srcpad_buffer_pool;
-#endif
 
   GstVaapiDisplay *display;
   GstVaapiDisplayType display_type;
   GstVaapiDisplayType display_type_req;
   gchar *display_name;
 
+  GstObject *gl_context;
+
   GstVaapiUploader *uploader;
   gboolean uploader_used;
 };
@@ -190,6 +192,11 @@ G_GNUC_INTERNAL
 void
 gst_vaapi_plugin_base_close (GstVaapiPluginBase * plugin);
 
+G_GNUC_INTERNAL
+gboolean
+gst_vaapi_plugin_base_has_display_type (GstVaapiPluginBase * plugin,
+    GstVaapiDisplayType display_type_req);
+
 G_GNUC_INTERNAL
 void
 gst_vaapi_plugin_base_set_display_type (GstVaapiPluginBase * plugin,
@@ -233,6 +240,11 @@ GstFlowReturn
 gst_vaapi_plugin_base_get_input_buffer (GstVaapiPluginBase * plugin,
     GstBuffer * inbuf, GstBuffer ** outbuf_ptr);
 
+G_GNUC_INTERNAL
+void
+gst_vaapi_plugin_base_set_gl_context (GstVaapiPluginBase * plugin,
+    GstObject * object);
+
 G_END_DECLS
 
 #endif /* GST_VAAPI_PLUGIN_BASE_H */
