@@ -29,16 +29,14 @@
  * implemented.
  */
 
-#include "gst/vaapi/sysdeps.h"
+#include "gstcompat.h"
 #include <gst/video/video.h>
 
 #include "gstvaapipostproc.h"
 #include "gstvaapipluginutil.h"
 #include "gstvaapivideobuffer.h"
-#if GST_CHECK_VERSION(1,0,0)
 #include "gstvaapivideobufferpool.h"
 #include "gstvaapivideomemory.h"
-#endif
 
 #define GST_PLUGIN_NAME "vaapipostproc"
 #define GST_PLUGIN_DESC "A video postprocessing filter"
@@ -46,28 +44,16 @@
 GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapipostproc);
 #define GST_CAT_DEFAULT gst_debug_vaapipostproc
 
-#if GST_CHECK_VERSION(1,1,0)
 # define GST_VAAPIPOSTPROC_SURFACE_CAPS \
     GST_VIDEO_CAPS_MAKE_WITH_FEATURES(  \
         GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE, "{ ENCODED, I420, YV12, NV12 }")
-#else
-# define GST_VAAPIPOSTPROC_SURFACE_CAPS \
-    GST_VAAPI_SURFACE_CAPS
-#endif
 
 /* Default templates */
 /* *INDENT-OFF* */
 static const char gst_vaapipostproc_sink_caps_str[] =
   GST_VAAPIPOSTPROC_SURFACE_CAPS ", "
   GST_CAPS_INTERLACED_MODES "; "
-#if GST_CHECK_VERSION(1,0,0)
   GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
-#else
-  "video/x-raw-yuv, "
-  "width  = " GST_VIDEO_SIZE_RANGE ", "
-  "height = " GST_VIDEO_SIZE_RANGE ", "
-  "framerate = " GST_VIDEO_FPS_RANGE ", "
-#endif
    GST_CAPS_INTERLACED_MODES;
 /* *INDENT-ON* */
 
@@ -75,19 +61,10 @@ static const char gst_vaapipostproc_sink_caps_str[] =
 static const char gst_vaapipostproc_src_caps_str[] =
   GST_VAAPIPOSTPROC_SURFACE_CAPS ", "
   GST_CAPS_INTERLACED_FALSE "; "
-#if GST_CHECK_VERSION(1,1,0)
   GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
-      GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "RGBA") ", "
+      GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "{ RGBA, BGRA }") ", "
   GST_CAPS_INTERLACED_FALSE "; "
-#endif
-#if GST_CHECK_VERSION(1,0,0)
   GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
-#else
-  "video/x-raw-yuv, "
-  "width  = " GST_VIDEO_SIZE_RANGE ", "
-  "height = " GST_VIDEO_SIZE_RANGE ", "
-  "framerate = " GST_VIDEO_FPS_RANGE ", "
-#endif
   GST_CAPS_INTERLACED_FALSE;
 /* *INDENT-ON* */
 
@@ -107,8 +84,15 @@ static GstStaticPadTemplate gst_vaapipostproc_src_factory =
     GST_STATIC_CAPS (gst_vaapipostproc_src_caps_str));
 /* *INDENT-ON* */
 
+static void gst_vaapipostproc_colorbalance_init (gpointer iface, gpointer data);
+
 G_DEFINE_TYPE_WITH_CODE (GstVaapiPostproc, gst_vaapipostproc,
-    GST_TYPE_BASE_TRANSFORM, GST_VAAPI_PLUGIN_BASE_INIT_INTERFACES);
+    GST_TYPE_BASE_TRANSFORM, GST_VAAPI_PLUGIN_BASE_INIT_INTERFACES
+    G_IMPLEMENT_INTERFACE (GST_TYPE_COLOR_BALANCE,
+        gst_vaapipostproc_colorbalance_init));
+
+static GstVideoFormat native_formats[] =
+    { GST_VIDEO_FORMAT_NV12, GST_VIDEO_FORMAT_YV12, GST_VIDEO_FORMAT_I420 };
 
 enum
 {
@@ -127,6 +111,7 @@ enum
   PROP_BRIGHTNESS,
   PROP_CONTRAST,
   PROP_SCALE_METHOD,
+  PROP_SKIN_TONE_ENHANCEMENT,
 };
 
 #define DEFAULT_FORMAT                  GST_VIDEO_FORMAT_ENCODED
@@ -247,6 +232,9 @@ gst_vaapipostproc_ensure_filter (GstVaapiPostproc * postproc)
   if (!gst_vaapipostproc_ensure_display (postproc))
     return FALSE;
 
+  gst_caps_replace (&postproc->allowed_srcpad_caps, NULL);
+  gst_caps_replace (&postproc->allowed_sinkpad_caps, NULL);
+
   postproc->filter =
       gst_vaapi_filter_new (GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc));
   if (!postproc->filter)
@@ -297,9 +285,12 @@ gst_vaapipostproc_destroy_filter (GstVaapiPostproc * postproc)
     g_ptr_array_unref (postproc->filter_ops);
     postproc->filter_ops = NULL;
   }
+  if (postproc->cb_channels) {
+    g_list_free_full (postproc->cb_channels, g_object_unref);
+    postproc->cb_channels = NULL;
+  }
   gst_vaapi_filter_replace (&postproc->filter, NULL);
   gst_vaapi_video_pool_replace (&postproc->filter_pool, NULL);
-  postproc->filter_pool_active = FALSE;
 }
 
 static void
@@ -321,7 +312,7 @@ gst_vaapipostproc_start (GstBaseTransform * trans)
   ds_reset (&postproc->deinterlace_state);
   if (!gst_vaapi_plugin_base_open (GST_VAAPI_PLUGIN_BASE (postproc)))
     return FALSE;
-  if (!gst_vaapipostproc_ensure_display (postproc))
+  if (!gst_vaapipostproc_ensure_filter (postproc))
     return FALSE;
   return TRUE;
 }
@@ -333,12 +324,11 @@ gst_vaapipostproc_stop (GstBaseTransform * trans)
 
   ds_reset (&postproc->deinterlace_state);
   gst_vaapi_plugin_base_close (GST_VAAPI_PLUGIN_BASE (postproc));
-  postproc->filter_pool_active = FALSE;
 
   postproc->field_duration = GST_CLOCK_TIME_NONE;
-  gst_video_info_init(&postproc->sinkpad_info);
-  gst_video_info_init(&postproc->srcpad_info);
-  gst_video_info_init(&postproc->filter_pool_info);
+  gst_video_info_init (&postproc->sinkpad_info);
+  gst_video_info_init (&postproc->srcpad_info);
+  gst_video_info_init (&postproc->filter_pool_info);
 
   return TRUE;
 }
@@ -361,13 +351,8 @@ should_deinterlace_buffer (GstVaapiPostproc * postproc, GstBuffer * buf)
     case GST_VIDEO_INTERLACE_MODE_PROGRESSIVE:
       return FALSE;
     case GST_VIDEO_INTERLACE_MODE_MIXED:
-#if GST_CHECK_VERSION(1,0,0)
       if (GST_BUFFER_FLAG_IS_SET (buf, GST_VIDEO_BUFFER_FLAG_INTERLACED))
         return TRUE;
-#else
-      if (!GST_BUFFER_FLAG_IS_SET (buf, GST_VIDEO_BUFFER_PROGRESSIVE))
-        return TRUE;
-#endif
       break;
     default:
       GST_ERROR ("unhandled \"interlace-mode\", disabling deinterlacing");
@@ -381,42 +366,27 @@ create_output_buffer (GstVaapiPostproc * postproc)
 {
   GstBuffer *outbuf;
 
-#if GST_CHECK_VERSION(1,0,0)
   GstBufferPool *const pool =
       GST_VAAPI_PLUGIN_BASE (postproc)->srcpad_buffer_pool;
   GstFlowReturn ret;
 
   g_return_val_if_fail (pool != NULL, NULL);
 
-  if (!postproc->filter_pool_active) {
-    if (!gst_buffer_pool_set_active (pool, TRUE))
-      goto error_activate_pool;
-    postproc->filter_pool_active = TRUE;
-  }
+  if (!gst_buffer_pool_set_active (pool, TRUE))
+    goto error_activate_pool;
 
   outbuf = NULL;
   ret = gst_buffer_pool_acquire_buffer (pool, &outbuf, NULL);
   if (ret != GST_FLOW_OK || !outbuf)
     goto error_create_buffer;
-#else
-  /* Create a raw VA video buffer without GstVaapiVideoMeta attached
-     to it yet, as this will be done next in the transform() hook */
-  outbuf = gst_vaapi_video_buffer_new_empty ();
-  if (!outbuf)
-    goto error_create_buffer;
-
-  gst_buffer_set_caps (outbuf, GST_VAAPI_PLUGIN_BASE_SRC_PAD_CAPS (postproc));
-#endif
   return outbuf;
 
   /* ERRORS */
-#if GST_CHECK_VERSION(1,0,0)
 error_activate_pool:
   {
     GST_ERROR ("failed to activate output video buffer pool");
     return NULL;
   }
-#endif
 error_create_buffer:
   {
     GST_ERROR ("failed to create output video buffer");
@@ -434,7 +404,6 @@ append_output_buffer_metadata (GstVaapiPostproc * postproc, GstBuffer * outbuf,
   gst_buffer_copy_into (outbuf, inbuf, flags | GST_BUFFER_COPY_FLAGS, 0, -1);
 
   /* GstVideoCropMeta */
-#if GST_CHECK_VERSION(1,0,0)
   if (!postproc->use_vpp) {
     GstVideoCropMeta *const crop_meta = gst_buffer_get_video_crop_meta (inbuf);
     if (crop_meta) {
@@ -444,7 +413,6 @@ append_output_buffer_metadata (GstVaapiPostproc * postproc, GstBuffer * outbuf,
         *out_crop_meta = *crop_meta;
     }
   }
-#endif
 
   /* GstVaapiVideoMeta */
   inbuf_meta = gst_buffer_get_vaapi_video_meta (inbuf);
@@ -529,9 +497,7 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
   guint flags, deint_flags;
   gboolean tff, deint, deint_refs, deint_changed;
   GstVaapiRectangle *crop_rect = NULL;
-#if GST_CHECK_VERSION(1,0,0)
   GstVaapiRectangle tmp_rect;
-#endif
 
   /* Validate filters */
   if ((postproc->flags & GST_VAAPI_POSTPROC_FLAG_FORMAT) &&
@@ -568,12 +534,16 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
       !gst_vaapi_filter_set_scaling (postproc->filter, postproc->scale_method))
     return GST_FLOW_NOT_SUPPORTED;
 
+  if ((postproc->flags & GST_VAAPI_POSTPROC_FLAG_SKINTONE) &&
+      !gst_vaapi_filter_set_skintone (postproc->filter,
+          postproc->skintone_enhance))
+    return GST_FLOW_NOT_SUPPORTED;
+
   inbuf_meta = gst_buffer_get_vaapi_video_meta (inbuf);
   if (!inbuf_meta)
     goto error_invalid_buffer;
   inbuf_surface = gst_vaapi_video_meta_get_surface (inbuf_meta);
 
-#if GST_CHECK_VERSION(1,0,0)
   GstVideoCropMeta *const crop_meta = gst_buffer_get_video_crop_meta (inbuf);
   if (crop_meta) {
     crop_rect = &tmp_rect;
@@ -582,7 +552,6 @@ gst_vaapipostproc_process_vpp (GstBaseTransform * trans, GstBuffer * inbuf,
     crop_rect->width = crop_meta->width;
     crop_rect->height = crop_meta->height;
   }
-#endif
   if (!crop_rect)
     crop_rect = (GstVaapiRectangle *)
         gst_vaapi_video_meta_get_render_rect (inbuf_meta);
@@ -910,6 +879,8 @@ gst_vaapipostproc_update_sink_caps (GstVaapiPostproc * postproc, GstCaps * caps,
   GstVideoInfo vi;
   gboolean deinterlace;
 
+  GST_INFO_OBJECT (postproc, "new sink caps = %" GST_PTR_FORMAT, caps);
+
   if (!gst_video_info_from_caps (&vi, caps))
     return FALSE;
 
@@ -933,6 +904,8 @@ gst_vaapipostproc_update_src_caps (GstVaapiPostproc * postproc, GstCaps * caps,
 {
   GstVideoInfo vi;
 
+  GST_INFO_OBJECT (postproc, "new src caps = %" GST_PTR_FORMAT, caps);
+
   if (!gst_video_info_from_caps (&vi, caps))
     return FALSE;
 
@@ -987,10 +960,8 @@ expand_allowed_srcpad_caps (GstVaapiPostproc * postproc, GstCaps * caps)
 {
   GValue value = G_VALUE_INIT, v_format = G_VALUE_INIT;
   guint i, num_structures;
-  gboolean had_filter;
 
-  had_filter = postproc->filter != NULL;
-  if (!had_filter && !gst_vaapipostproc_ensure_filter (postproc))
+  if (postproc->filter == NULL)
     goto cleanup;
   if (!gst_vaapipostproc_ensure_filter_caps (postproc))
     goto cleanup;
@@ -1005,12 +976,10 @@ expand_allowed_srcpad_caps (GstVaapiPostproc * postproc, GstCaps * caps)
 
   num_structures = gst_caps_get_size (caps);
   for (i = 0; i < num_structures; i++) {
-#if GST_CHECK_VERSION(1,1,0)
     GstCapsFeatures *const features = gst_caps_get_features (caps, i);
     if (gst_caps_features_contains (features,
             GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META))
       continue;
-#endif
 
     GstStructure *const structure = gst_caps_get_structure (caps, i);
     if (!structure)
@@ -1020,8 +989,6 @@ expand_allowed_srcpad_caps (GstVaapiPostproc * postproc, GstCaps * caps)
   g_value_unset (&value);
 
 cleanup:
-  if (!had_filter)
-    gst_vaapipostproc_destroy_filter (postproc);
   return caps;
 }
 
@@ -1082,10 +1049,8 @@ gst_vaapipostproc_transform_caps_impl (GstBaseTransform * trans,
   GstVideoInfo vi;
   GstVideoFormat out_format;
   GstCaps *out_caps;
-#if GST_CHECK_VERSION(1,1,0)
   GstVaapiCapsFeature feature;
   const gchar *feature_str;
-#endif
   guint width, height;
 
   /* Generate the sink pad caps, that could be fixated afterwards */
@@ -1117,7 +1082,6 @@ gst_vaapipostproc_transform_caps_impl (GstBaseTransform * trans,
     GST_VIDEO_INFO_FPS_N (&vi) = fps_n;
     GST_VIDEO_INFO_FPS_D (&vi) = fps_d;
   }
-
   // Signal the other pad that we only generate progressive frames
   GST_VIDEO_INFO_INTERLACE_MODE (&vi) = GST_VIDEO_INTERLACE_MODE_PROGRESSIVE;
 
@@ -1125,7 +1089,6 @@ gst_vaapipostproc_transform_caps_impl (GstBaseTransform * trans,
   find_best_size (postproc, &vi, &width, &height);
 
   // Update format from user-specified parameters
-#if GST_CHECK_VERSION(1,1,0)
   /* XXX: this is a workaround until auto-plugging is fixed when
    * format=ENCODED + memory:VASurface caps feature are provided.
    * use the downstream negotiated video format as the output format
@@ -1152,20 +1115,9 @@ gst_vaapipostproc_transform_caps_impl (GstBaseTransform * trans,
 
   feature =
       gst_vaapi_find_preferred_caps_feature (GST_BASE_TRANSFORM_SRC_PAD (trans),
-      out_format);
-  switch (feature) {
-    case GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META:
-      out_format = GST_VIDEO_FORMAT_RGBA;
-      break;
-    default:
-      break;
-  }
-#else
-  out_format = GST_VIDEO_FORMAT_ENCODED;
-#endif
+      out_format, &out_format);
   gst_video_info_change_format (&vi, out_format, width, height);
 
-#if GST_CHECK_VERSION(1,1,0)
   out_caps = gst_video_info_to_caps (&vi);
   if (!out_caps)
     return NULL;
@@ -1176,29 +1128,16 @@ gst_vaapipostproc_transform_caps_impl (GstBaseTransform * trans,
       gst_caps_set_features (out_caps, 0,
           gst_caps_features_new (feature_str, NULL));
   }
-#else
-  /* XXX: gst_video_info_to_caps() from GStreamer 0.10 does not
-     reconstruct suitable caps for "encoded" video formats */
-  out_caps = gst_caps_from_string (GST_VAAPI_SURFACE_CAPS_NAME);
-  if (!out_caps)
-    return NULL;
 
-  gst_caps_set_simple (out_caps,
-      "type", G_TYPE_STRING, "vaapi",
-      "opengl", G_TYPE_BOOLEAN, USE_GLX,
-      "width", G_TYPE_INT, GST_VIDEO_INFO_WIDTH (&vi),
-      "height", G_TYPE_INT, GST_VIDEO_INFO_HEIGHT (&vi),
-      "framerate", GST_TYPE_FRACTION, GST_VIDEO_INFO_FPS_N (&vi),
-      GST_VIDEO_INFO_FPS_D (&vi),
-      "pixel-aspect-ratio", GST_TYPE_FRACTION, GST_VIDEO_INFO_PAR_N (&vi),
-      GST_VIDEO_INFO_PAR_D (&vi), NULL);
-
-  gst_caps_set_interlaced (out_caps, &vi);
-#endif
+  /* we don't need to do format conversion if GL_TEXTURE_UPLOAD_META
+   * is negotiated */
+  if (feature != GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META &&
+      postproc->format != out_format) {
+    postproc->format = out_format;
+  }
   return out_caps;
 }
 
-#if GST_CHECK_VERSION(1,0,0)
 static GstCaps *
 gst_vaapipostproc_transform_caps (GstBaseTransform * trans,
     GstPadDirection direction, GstCaps * caps, GstCaps * filter)
@@ -1213,21 +1152,11 @@ gst_vaapipostproc_transform_caps (GstBaseTransform * trans,
   }
   return caps;
 }
-#else
-#define gst_vaapipostproc_transform_caps \
-    gst_vaapipostproc_transform_caps_impl
-#endif
-
-#if GST_CHECK_VERSION(1,0,0)
-typedef gsize GstBaseTransformSizeType;
-#else
-typedef guint GstBaseTransformSizeType;
-#endif
 
 static gboolean
 gst_vaapipostproc_transform_size (GstBaseTransform * trans,
-    GstPadDirection direction, GstCaps * caps, GstBaseTransformSizeType size,
-    GstCaps * othercaps, GstBaseTransformSizeType * othersize)
+    GstPadDirection direction, GstCaps * caps, gsize size,
+    GstCaps * othercaps, gsize * othersize)
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
 
@@ -1282,11 +1211,7 @@ done:
 
 static GstFlowReturn
 gst_vaapipostproc_prepare_output_buffer (GstBaseTransform * trans,
-    GstBuffer * inbuf,
-#if !GST_CHECK_VERSION(1,0,0)
-    gint size, GstCaps * caps,
-#endif
-    GstBuffer ** outbuf_ptr)
+    GstBuffer * inbuf, GstBuffer ** outbuf_ptr)
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
 
@@ -1321,15 +1246,37 @@ ensure_srcpad_buffer_pool (GstVaapiPostproc * postproc, GstCaps * caps)
   return TRUE;
 }
 
+static gboolean
+is_native_video_format (GstVideoFormat format)
+{
+  guint i = 0;
+  for (i = 0; i < G_N_ELEMENTS (native_formats); i++)
+    if (native_formats[i] == format)
+      return TRUE;
+  return FALSE;
+}
+
 static gboolean
 gst_vaapipostproc_set_caps (GstBaseTransform * trans, GstCaps * caps,
     GstCaps * out_caps)
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
   gboolean caps_changed = FALSE;
+  GstVideoInfo vinfo;
 
   if (!gst_vaapipostproc_update_sink_caps (postproc, caps, &caps_changed))
     return FALSE;
+  /* HACK: This is a workaround to deal with the va-intel-driver for non-native
+   * formats while doing advanced deinterlacing. The format of reference surfaces must
+   * be same as the format used by the driver internally for motion adaptive
+   * deinterlacing and motion compensated deinterlacing */
+  gst_video_info_from_caps (&vinfo, caps);
+  if (deint_method_is_advanced (postproc->deinterlace_method)
+      && !is_native_video_format (GST_VIDEO_INFO_FORMAT (&vinfo))) {
+    GST_WARNING_OBJECT (postproc,
+        "Advanced deinterlacing requires the native video formats used by the driver internally");
+    return FALSE;
+  }
   if (!gst_vaapipostproc_update_src_caps (postproc, out_caps, &caps_changed))
     return FALSE;
 
@@ -1353,11 +1300,10 @@ gst_vaapipostproc_query (GstBaseTransform * trans, GstPadDirection direction,
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
 
-  GST_INFO_OBJECT (trans, "query type `%s'", GST_QUERY_TYPE_NAME (query));
-
   if (gst_vaapi_reply_to_query (query,
           GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc))) {
-    GST_DEBUG ("sharing display %p", GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc));
+    GST_DEBUG_OBJECT (postproc, "sharing display %p",
+        GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc));
     return TRUE;
   }
 
@@ -1366,7 +1312,6 @@ gst_vaapipostproc_query (GstBaseTransform * trans, GstPadDirection direction,
       direction, query);
 }
 
-#if GST_CHECK_VERSION(1,0,0)
 static gboolean
 gst_vaapipostproc_propose_allocation (GstBaseTransform * trans,
     GstQuery * decide_query, GstQuery * query)
@@ -1388,7 +1333,6 @@ gst_vaapipostproc_decide_allocation (GstBaseTransform * trans, GstQuery * query)
   return gst_vaapi_plugin_base_decide_allocation (GST_VAAPI_PLUGIN_BASE (trans),
       query, 0);
 }
-#endif
 
 static void
 gst_vaapipostproc_finalize (GObject * object)
@@ -1454,6 +1398,10 @@ gst_vaapipostproc_set_property (GObject * object,
       postproc->scale_method = g_value_get_enum (value);
       postproc->flags |= GST_VAAPI_POSTPROC_FLAG_SCALE;
       break;
+    case PROP_SKIN_TONE_ENHANCEMENT:
+      postproc->skintone_enhance = g_value_get_boolean (value);
+      postproc->flags |= GST_VAAPI_POSTPROC_FLAG_SKINTONE;
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1506,6 +1454,9 @@ gst_vaapipostproc_get_property (GObject * object,
     case PROP_SCALE_METHOD:
       g_value_set_enum (value, postproc->scale_method);
       break;
+    case PROP_SKIN_TONE_ENHANCEMENT:
+      g_value_set_boolean (value, postproc->skintone_enhance);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1537,11 +1488,8 @@ gst_vaapipostproc_class_init (GstVaapiPostprocClass * klass)
   trans_class->transform = gst_vaapipostproc_transform;
   trans_class->set_caps = gst_vaapipostproc_set_caps;
   trans_class->query = gst_vaapipostproc_query;
-
-#if GST_CHECK_VERSION(1,0,0)
   trans_class->propose_allocation = gst_vaapipostproc_propose_allocation;
   trans_class->decide_allocation = gst_vaapipostproc_decide_allocation;
-#endif
 
   trans_class->prepare_output_buffer = gst_vaapipostproc_prepare_output_buffer;
 
@@ -1724,12 +1672,60 @@ gst_vaapipostproc_class_init (GstVaapiPostprocClass * klass)
     g_object_class_install_property (object_class,
         PROP_SCALE_METHOD, filter_op->pspec);
 
+  /**
+   * GstVaapiPostproc:skin-tone-enhancement:
+   *
+   * Apply the skin tone enhancement algorithm.
+   */
+  filter_op = find_filter_op (filter_ops, GST_VAAPI_FILTER_OP_SKINTONE);
+  if (filter_op)
+    g_object_class_install_property (object_class,
+        PROP_SKIN_TONE_ENHANCEMENT, filter_op->pspec);
+
   g_ptr_array_unref (filter_ops);
 }
 
+static float *
+find_value_ptr (GstVaapiPostproc * postproc, GstVaapiFilterOp op)
+{
+  switch (op) {
+    case GST_VAAPI_FILTER_OP_HUE:
+      return &postproc->hue;
+    case GST_VAAPI_FILTER_OP_SATURATION:
+      return &postproc->saturation;
+    case GST_VAAPI_FILTER_OP_BRIGHTNESS:
+      return &postproc->brightness;
+    case GST_VAAPI_FILTER_OP_CONTRAST:
+      return &postproc->contrast;
+    default:
+      return NULL;
+  }
+}
+
+static void
+cb_set_default_value (GstVaapiPostproc * postproc, GPtrArray * filter_ops,
+    GstVaapiFilterOp op)
+{
+  GstVaapiFilterOpInfo *filter_op;
+  GParamSpecFloat *pspec;
+  float *var;
+
+  filter_op = find_filter_op (filter_ops, op);
+  if (!filter_op)
+    return;
+  var = find_value_ptr (postproc, op);
+  if (!var)
+    return;
+  pspec = G_PARAM_SPEC_FLOAT (filter_op->pspec);
+  *var = pspec->default_value;
+}
+
 static void
 gst_vaapipostproc_init (GstVaapiPostproc * postproc)
 {
+  GPtrArray *filter_ops;
+  guint i;
+
   gst_vaapi_plugin_base_init (GST_VAAPI_PLUGIN_BASE (postproc),
       GST_CAT_DEFAULT);
 
@@ -1740,7 +1736,156 @@ gst_vaapipostproc_init (GstVaapiPostproc * postproc)
   postproc->keep_aspect = TRUE;
   postproc->get_va_surfaces = TRUE;
 
+  filter_ops = gst_vaapi_filter_get_operations (NULL);
+  if (filter_ops) {
+    for (i = GST_VAAPI_FILTER_OP_HUE; i <= GST_VAAPI_FILTER_OP_CONTRAST; i++)
+      cb_set_default_value (postproc, filter_ops, i);
+    g_ptr_array_unref (filter_ops);
+  }
+
   gst_video_info_init (&postproc->sinkpad_info);
   gst_video_info_init (&postproc->srcpad_info);
   gst_video_info_init (&postproc->filter_pool_info);
 }
+
+/* ------------------------------------------------------------------------ */
+/* --- GstColorBalance interface                                        --- */
+/* ------------------------------------------------------------------------ */
+
+#define CB_CHANNEL_FACTOR 1000.0
+
+typedef struct
+{
+  GstVaapiFilterOp op;
+  const gchar *name;
+} ColorBalanceChannel;
+
+ColorBalanceChannel cb_channels[] = {
+  {GST_VAAPI_FILTER_OP_HUE, "VA_FILTER_HUE"},
+  {GST_VAAPI_FILTER_OP_SATURATION, "VA_FILTER_SATURATION"},
+  {GST_VAAPI_FILTER_OP_BRIGHTNESS, "VA_FILTER_BRIGHTNESS"},
+  {GST_VAAPI_FILTER_OP_CONTRAST, "VA_FILTER_CONTRAST"},
+};
+
+static void
+cb_channels_init (GstVaapiPostproc * postproc)
+{
+  GPtrArray *filter_ops;
+  GstVaapiFilterOpInfo *filter_op;
+  GParamSpecFloat *pspec;
+  GstColorBalanceChannel *channel;
+  guint i;
+
+  if (postproc->cb_channels)
+    return;
+
+  if (!gst_vaapipostproc_ensure_filter (postproc))
+    return;
+
+  filter_ops = postproc->filter_ops ? g_ptr_array_ref (postproc->filter_ops)
+      : gst_vaapi_filter_get_operations (postproc->filter);
+  if (!filter_ops)
+    return;
+
+  for (i = 0; i < G_N_ELEMENTS (cb_channels); i++) {
+    filter_op = find_filter_op (filter_ops, cb_channels[i].op);
+    if (!filter_op)
+      continue;
+
+    pspec = G_PARAM_SPEC_FLOAT (filter_op->pspec);
+    channel = g_object_new (GST_TYPE_COLOR_BALANCE_CHANNEL, NULL);
+    channel->label = g_strdup (cb_channels[i].name);
+    channel->min_value = pspec->minimum * CB_CHANNEL_FACTOR;
+    channel->max_value = pspec->maximum * CB_CHANNEL_FACTOR;
+
+    postproc->cb_channels = g_list_prepend (postproc->cb_channels, channel);
+  }
+
+  g_ptr_array_unref (filter_ops);
+}
+
+static const GList *
+gst_vaapipostproc_colorbalance_list_channels (GstColorBalance * balance)
+{
+  GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (balance);
+
+  cb_channels_init (postproc);
+  return postproc->cb_channels;
+}
+
+static gfloat *
+cb_get_value_ptr (GstVaapiPostproc * postproc, GstColorBalanceChannel * channel,
+    GstVaapiPostprocFlags * flags)
+{
+  guint i;
+  gfloat *ret = NULL;
+
+  for (i = 0; i < G_N_ELEMENTS (cb_channels); i++) {
+    if (g_ascii_strcasecmp (cb_channels[i].name, channel->label) == 0)
+      break;
+  }
+  if (i >= G_N_ELEMENTS (cb_channels))
+    return NULL;
+
+  ret = find_value_ptr (postproc, cb_channels[i].op);
+  if (flags)
+    *flags = 1 << cb_channels[i].op;
+  return ret;
+}
+
+static void
+gst_vaapipostproc_colorbalance_set_value (GstColorBalance * balance,
+    GstColorBalanceChannel * channel, gint value)
+{
+  GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (balance);
+  GstVaapiPostprocFlags flags;
+  gfloat new_val, *var;
+
+  value = CLAMP (value, channel->min_value, channel->max_value);
+  new_val = (gfloat) value / CB_CHANNEL_FACTOR;
+
+  var = cb_get_value_ptr (postproc, channel, &flags);
+  if (var) {
+    *var = new_val;
+    postproc->flags |= flags;
+    gst_color_balance_value_changed (balance, channel, value);
+    return;
+  }
+
+  GST_WARNING_OBJECT (postproc, "unknown channel %s", channel->label);
+}
+
+static gint
+gst_vaapipostproc_colorbalance_get_value (GstColorBalance * balance,
+    GstColorBalanceChannel * channel)
+{
+  GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (balance);
+  gfloat *var;
+  gint new_val;
+
+  var = cb_get_value_ptr (postproc, channel, NULL);
+  if (var) {
+    new_val = (gint) ((*var) * CB_CHANNEL_FACTOR);
+    new_val = CLAMP (new_val, channel->min_value, channel->max_value);
+    return new_val;
+  }
+
+  GST_WARNING_OBJECT (postproc, "unknown channel %s", channel->label);
+  return G_MININT;
+}
+
+static GstColorBalanceType
+gst_vaapipostproc_colorbalance_get_balance_type (GstColorBalance * balance)
+{
+  return GST_COLOR_BALANCE_HARDWARE;
+}
+
+static void
+gst_vaapipostproc_colorbalance_init (gpointer iface, gpointer data)
+{
+  GstColorBalanceInterface *cbface = iface;
+  cbface->list_channels = gst_vaapipostproc_colorbalance_list_channels;
+  cbface->set_value = gst_vaapipostproc_colorbalance_set_value;
+  cbface->get_value = gst_vaapipostproc_colorbalance_get_value;
+  cbface->get_balance_type = gst_vaapipostproc_colorbalance_get_balance_type;
+}
