@@ -31,41 +31,22 @@
  * create its own internal window and render into it.
  */
 
-#include "gst/vaapi/sysdeps.h"
+#include "gstcompat.h"
 #include <gst/gst.h>
 #include <gst/video/video.h>
 
 #include <gst/vaapi/gstvaapivalue.h>
 
 /* Supported interfaces */
-#if GST_CHECK_VERSION(1,0,0)
 # include <gst/video/videooverlay.h>
 # include <gst/video/colorbalance.h>
 # include <gst/video/navigation.h>
-#else
-# include <gst/interfaces/xoverlay.h>
-# include <gst/interfaces/colorbalance.h>
-
-# define GST_TYPE_VIDEO_OVERLAY         GST_TYPE_X_OVERLAY
-# define GST_VIDEO_OVERLAY              GST_X_OVERLAY
-# define GstVideoOverlay                GstXOverlay
-# define GstVideoOverlayInterface       GstXOverlayClass
-
-# define gst_video_overlay_prepare_window_handle(sink) \
-    gst_x_overlay_prepare_xwindow_id(sink)
-# define gst_video_overlay_got_window_handle(sink, window_handle) \
-    gst_x_overlay_got_window_handle(sink, window_handle)
-
-# define GstColorBalanceInterface       GstColorBalanceClass
-#endif
 
 #include "gstvaapisink.h"
 #include "gstvaapipluginutil.h"
 #include "gstvaapivideometa.h"
-#if GST_CHECK_VERSION(1,0,0)
 #include "gstvaapivideobufferpool.h"
 #include "gstvaapivideomemory.h"
-#endif
 
 #define GST_PLUGIN_NAME "vaapisink"
 #define GST_PLUGIN_DESC "A VA-API based videosink"
@@ -76,20 +57,19 @@ GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapisink);
 /* Default template */
 /* *INDENT-OFF* */
 static const char gst_vaapisink_sink_caps_str[] =
-#if GST_CHECK_VERSION(1,1,0)
-    GST_VIDEO_CAPS_MAKE_WITH_FEATURES (GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE,
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
+        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE,
         "{ ENCODED, NV12, I420, YV12 }") ";"
+#if GST_CHECK_VERSION(1,3,1)
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
+        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE ","
+            GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION,
+        "{ ENCODED, NV12, I420, YV12 }") ";"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
+        GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION,
+        GST_VIDEO_FORMATS_ALL) ";"
+#endif
     GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL);
-#else
-#if GST_CHECK_VERSION(1,0,0)
-    GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) "; "
-#else
-    "video/x-raw-yuv, "
-    "width  = (int) [ 1, MAX ], "
-    "height = (int) [ 1, MAX ]; "
-#endif
-    GST_VAAPI_SURFACE_CAPS;
-#endif
 /* *INDENT-ON* */
 
 static GstStaticPadTemplate gst_vaapisink_sink_factory =
@@ -124,6 +104,14 @@ G_DEFINE_TYPE_WITH_CODE (GstVaapiSink,
     G_IMPLEMENT_INTERFACE (GST_TYPE_NAVIGATION,
         gst_vaapisink_navigation_iface_init));
 
+enum
+{
+  HANDOFF_SIGNAL,
+  LAST_SIGNAL
+};
+
+static guint gst_vaapisink_signals[LAST_SIGNAL] = { 0 };
+
 enum
 {
   PROP_0,
@@ -138,26 +126,26 @@ enum
   PROP_SATURATION,
   PROP_BRIGHTNESS,
   PROP_CONTRAST,
+  PROP_SIGNAL_HANDOFFS,
 
   N_PROPERTIES
 };
 
 #define DEFAULT_DISPLAY_TYPE            GST_VAAPI_DISPLAY_TYPE_ANY
 #define DEFAULT_ROTATION                GST_VAAPI_ROTATION_0
+#define DEFAULT_SIGNAL_HANDOFFS         FALSE
 
 static GParamSpec *g_properties[N_PROPERTIES] = { NULL, };
 
-static void
-gst_vaapisink_video_overlay_expose (GstVideoOverlay * overlay);
+static void gst_vaapisink_video_overlay_expose (GstVideoOverlay * overlay);
 
-static gboolean
-gst_vaapisink_reconfigure_window (GstVaapiSink * sink);
+static gboolean gst_vaapisink_reconfigure_window (GstVaapiSink * sink);
 
 static void
 gst_vaapisink_set_event_handling (GstVaapiSink * sink, gboolean handle_events);
 
 static GstFlowReturn
-gst_vaapisink_show_frame (GstBaseSink * base_sink, GstBuffer * buffer);
+gst_vaapisink_show_frame (GstVideoSink * video_sink, GstBuffer * buffer);
 
 static gboolean
 gst_vaapisink_ensure_render_rect (GstVaapiSink * sink, guint width,
@@ -583,7 +571,7 @@ gst_vaapisink_video_overlay_expose (GstVideoOverlay * overlay)
 
   if (sink->video_buffer) {
     gst_vaapisink_reconfigure_window (sink);
-    gst_vaapisink_show_frame (GST_BASE_SINK_CAST (sink), sink->video_buffer);
+    gst_vaapisink_show_frame (GST_VIDEO_SINK_CAST (sink), sink->video_buffer);
   }
 }
 
@@ -831,11 +819,7 @@ gst_vaapisink_color_balance_iface_init (GstColorBalanceInterface * iface)
   iface->list_channels = gst_vaapisink_color_balance_list_channels;
   iface->set_value = gst_vaapisink_color_balance_set_value;
   iface->get_value = gst_vaapisink_color_balance_get_value;
-#if GST_CHECK_VERSION(1,0,0)
   iface->get_balance_type = gst_vaapisink_color_balance_get_type;
-#else
-  iface->balance_type = gst_vaapisink_color_balance_get_type (NULL);
-#endif
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1232,11 +1216,7 @@ gst_vaapisink_get_caps_impl (GstBaseSink * base_sink)
   GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
   GstCaps *out_caps, *yuv_caps;
 
-#if GST_CHECK_VERSION(1,1,0)
   out_caps = gst_static_pad_template_get_caps (&gst_vaapisink_sink_factory);
-#else
-  out_caps = gst_caps_from_string (GST_VAAPI_SURFACE_CAPS);
-#endif
   if (!out_caps)
     return NULL;
 
@@ -1250,7 +1230,6 @@ gst_vaapisink_get_caps_impl (GstBaseSink * base_sink)
   return out_caps;
 }
 
-#if GST_CHECK_VERSION(1,0,0)
 static inline GstCaps *
 gst_vaapisink_get_caps (GstBaseSink * base_sink, GstCaps * filter)
 {
@@ -1264,14 +1243,10 @@ gst_vaapisink_get_caps (GstBaseSink * base_sink, GstCaps * filter)
     out_caps = caps;
   return out_caps;
 }
-#else
-#define gst_vaapisink_get_caps gst_vaapisink_get_caps_impl
-#endif
 
 static void
 update_colorimetry (GstVaapiSink * sink, GstVideoColorimetry * cinfo)
 {
-#if GST_CHECK_VERSION(1,0,0)
   if (gst_video_colorimetry_matches (cinfo, GST_VIDEO_COLORIMETRY_BT601))
     sink->color_standard = GST_VAAPI_COLOR_STANDARD_ITUR_BT_601;
   else if (gst_video_colorimetry_matches (cinfo, GST_VIDEO_COLORIMETRY_BT709))
@@ -1282,7 +1257,12 @@ update_colorimetry (GstVaapiSink * sink, GstVideoColorimetry * cinfo)
   else
     sink->color_standard = 0;
 
-  GST_DEBUG ("colorimetry %s", gst_video_colorimetry_to_string (cinfo));
+#ifndef GST_DISABLE_GST_DEBUG
+  {
+    gchar *const colorimetry_string = gst_video_colorimetry_to_string (cinfo);
+    GST_DEBUG ("colorimetry %s", colorimetry_string);
+    g_free (colorimetry_string);
+  }
 #endif
 }
 
@@ -1351,13 +1331,10 @@ gst_vaapisink_show_frame_unlocked (GstVaapiSink * sink, GstBuffer * src_buffer)
   GstBuffer *buffer;
   guint flags;
   GstVaapiRectangle *surface_rect = NULL;
-#if GST_CHECK_VERSION(1,0,0)
   GstVaapiRectangle tmp_rect;
-#endif
   GstFlowReturn ret;
   gint32 view_id;
 
-#if GST_CHECK_VERSION(1,0,0)
   GstVideoCropMeta *const crop_meta =
       gst_buffer_get_video_crop_meta (src_buffer);
   if (crop_meta) {
@@ -1367,7 +1344,6 @@ gst_vaapisink_show_frame_unlocked (GstVaapiSink * sink, GstBuffer * src_buffer)
     surface_rect->width = crop_meta->width;
     surface_rect->height = crop_meta->height;
   }
-#endif
 
   ret = gst_vaapi_plugin_base_get_input_buffer (GST_VAAPI_PLUGIN_BASE (sink),
       src_buffer, &buffer);
@@ -1380,19 +1356,19 @@ gst_vaapisink_show_frame_unlocked (GstVaapiSink * sink, GstBuffer * src_buffer)
 
   proxy = gst_vaapi_video_meta_get_surface_proxy (meta);
   if (!proxy)
-    goto error;
+    goto no_surface;
 
   surface = gst_vaapi_video_meta_get_surface (meta);
   if (!surface)
-    goto error;
+    goto no_surface;
 
   /* Validate view component to display */
   view_id = GST_VAAPI_SURFACE_PROXY_VIEW_ID (proxy);
   if (G_UNLIKELY (sink->view_id == -1))
     sink->view_id = view_id;
   else if (sink->view_id != view_id) {
-    gst_buffer_unref (buffer);
-    return GST_FLOW_OK;
+    ret = GST_FLOW_OK;
+    goto done;
   }
 
   gst_vaapisink_ensure_colorbalance (sink);
@@ -1423,20 +1399,38 @@ gst_vaapisink_show_frame_unlocked (GstVaapiSink * sink, GstBuffer * src_buffer)
   if (!sink->backend->render_surface (sink, surface, surface_rect, flags))
     goto error;
 
+  if (sink->signal_handoffs)
+    g_signal_emit (sink, gst_vaapisink_signals[HANDOFF_SIGNAL], 0, buffer);
+
   /* Retain VA surface until the next one is displayed */
+  /* Need to release the lock for the duration, otherwise a deadlock is possible */
+  gst_vaapi_display_unlock (GST_VAAPI_PLUGIN_BASE_DISPLAY (sink));
   gst_buffer_replace (&sink->video_buffer, buffer);
+  gst_vaapi_display_lock (GST_VAAPI_PLUGIN_BASE_DISPLAY (sink));
+
+  ret = GST_FLOW_OK;
+
+done:
   gst_buffer_unref (buffer);
-  return GST_FLOW_OK;
+  return ret;
 
 error:
-  gst_buffer_unref (buffer);
-  return GST_FLOW_EOS;
+  GST_ELEMENT_ERROR (sink, RESOURCE, WRITE,
+      ("Internal error: could not render surface"), (NULL));
+  ret = GST_FLOW_ERROR;
+  goto done;
+
+no_surface:
+  /* No surface or surface proxy. That's very bad! */
+  GST_WARNING_OBJECT (sink, "could not get surface");
+  ret = GST_FLOW_ERROR;
+  goto done;
 }
 
 static GstFlowReturn
-gst_vaapisink_show_frame (GstBaseSink * base_sink, GstBuffer * src_buffer)
+gst_vaapisink_show_frame (GstVideoSink * video_sink, GstBuffer * src_buffer)
 {
-  GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
+  GstVaapiSink *const sink = GST_VAAPISINK_CAST (video_sink);
   GstFlowReturn ret;
 
   /* We need at least to protect the gst_vaapi_aplpy_composition()
@@ -1445,10 +1439,10 @@ gst_vaapisink_show_frame (GstBaseSink * base_sink, GstBuffer * src_buffer)
   gst_vaapi_display_lock (GST_VAAPI_PLUGIN_BASE_DISPLAY (sink));
   ret = gst_vaapisink_show_frame_unlocked (sink, src_buffer);
   gst_vaapi_display_unlock (GST_VAAPI_PLUGIN_BASE_DISPLAY (sink));
+
   return ret;
 }
 
-#if GST_CHECK_VERSION(1,0,0)
 static gboolean
 gst_vaapisink_propose_allocation (GstBaseSink * base_sink, GstQuery * query)
 {
@@ -1462,16 +1456,6 @@ gst_vaapisink_propose_allocation (GstBaseSink * base_sink, GstQuery * query)
       GST_VIDEO_OVERLAY_COMPOSITION_META_API_TYPE, NULL);
   return TRUE;
 }
-#else
-static GstFlowReturn
-gst_vaapisink_buffer_alloc (GstBaseSink * base_sink, guint64 offset, guint size,
-    GstCaps * caps, GstBuffer ** outbuf_ptr)
-{
-  return
-      gst_vaapi_plugin_base_allocate_input_buffer (GST_VAAPI_PLUGIN_BASE
-      (base_sink), caps, outbuf_ptr);
-}
-#endif
 
 static gboolean
 gst_vaapisink_query (GstBaseSink * base_sink, GstQuery * query)
@@ -1534,6 +1518,9 @@ gst_vaapisink_set_property (GObject * object,
     case PROP_FORCE_ASPECT_RATIO:
       sink->keep_aspect = g_value_get_boolean (value);
       break;
+    case PROP_SIGNAL_HANDOFFS:
+      sink->signal_handoffs = g_value_get_boolean (value);
+      break;
     case PROP_HUE:
     case PROP_SATURATION:
     case PROP_BRIGHTNESS:
@@ -1571,6 +1558,9 @@ gst_vaapisink_get_property (GObject * object,
     case PROP_FORCE_ASPECT_RATIO:
       g_value_set_boolean (value, sink->keep_aspect);
       break;
+    case PROP_SIGNAL_HANDOFFS:
+      g_value_set_boolean (value, sink->signal_handoffs);
+      break;
     case PROP_HUE:
     case PROP_SATURATION:
     case PROP_BRIGHTNESS:
@@ -1584,6 +1574,28 @@ gst_vaapisink_get_property (GObject * object,
   }
 }
 
+static gboolean
+gst_vaapisink_unlock (GstBaseSink * base_sink)
+{
+  GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
+
+  if (sink->window)
+    return gst_vaapi_window_unblock (sink->window);
+
+  return TRUE;
+}
+
+static gboolean
+gst_vaapisink_unlock_stop (GstBaseSink * base_sink)
+{
+  GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
+
+  if (sink->window)
+    return gst_vaapi_window_unblock_cancel (sink->window);
+
+  return TRUE;
+}
+
 static void
 gst_vaapisink_set_bus (GstElement * element, GstBus * bus)
 {
@@ -1603,6 +1615,7 @@ gst_vaapisink_class_init (GstVaapiSinkClass * klass)
   GObjectClass *const object_class = G_OBJECT_CLASS (klass);
   GstElementClass *const element_class = GST_ELEMENT_CLASS (klass);
   GstBaseSinkClass *const basesink_class = GST_BASE_SINK_CLASS (klass);
+  GstVideoSinkClass *const videosink_class = GST_VIDEO_SINK_CLASS (klass);
   GstVaapiPluginBaseClass *const base_plugin_class =
       GST_VAAPI_PLUGIN_BASE_CLASS (klass);
   GstPadTemplate *pad_template;
@@ -1622,14 +1635,12 @@ gst_vaapisink_class_init (GstVaapiSinkClass * klass)
   basesink_class->stop = gst_vaapisink_stop;
   basesink_class->get_caps = gst_vaapisink_get_caps;
   basesink_class->set_caps = gst_vaapisink_set_caps;
-  basesink_class->preroll = gst_vaapisink_show_frame;
-  basesink_class->render = gst_vaapisink_show_frame;
-  basesink_class->query = gst_vaapisink_query;
-#if GST_CHECK_VERSION(1,0,0)
+  basesink_class->query = GST_DEBUG_FUNCPTR (gst_vaapisink_query);
   basesink_class->propose_allocation = gst_vaapisink_propose_allocation;
-#else
-  basesink_class->buffer_alloc = gst_vaapisink_buffer_alloc;
-#endif
+  basesink_class->unlock = gst_vaapisink_unlock;
+  basesink_class->unlock_stop = gst_vaapisink_unlock_stop;
+
+  videosink_class->show_frame = GST_DEBUG_FUNCPTR (gst_vaapisink_show_frame);
 
   element_class->set_bus = gst_vaapisink_set_bus;
   gst_element_class_set_static_metadata (element_class,
@@ -1696,6 +1707,16 @@ gst_vaapisink_class_init (GstVaapiSinkClass * klass)
       "When enabled, scaling will respect original aspect ratio",
       TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GstVaapiSink:signal-handoffs:
+   *
+   * Send a signal after rendering the buffer.
+   */
+  g_properties[PROP_SIGNAL_HANDOFFS] =
+      g_param_spec_boolean ("signal-handoffs", "Signal handoffs",
+      "Send a signal after rendering the buffer", DEFAULT_SIGNAL_HANDOFFS,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * GstVaapiSink:view-id:
    *
@@ -1758,6 +1779,18 @@ gst_vaapisink_class_init (GstVaapiSinkClass * klass)
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 
   g_object_class_install_properties (object_class, N_PROPERTIES, g_properties);
+
+  /**
+   * GstVaapiSink::handoff:
+   * @object: the #GstVaapiSink instance
+   * @buffer: the buffer that was rendered
+   *
+   * This signal gets emitted after rendering the frame.
+   */
+  gst_vaapisink_signals[HANDOFF_SIGNAL] =
+      g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic,
+      G_TYPE_NONE, 1, GST_TYPE_BUFFER | G_SIGNAL_TYPE_STATIC_SCOPE);
 }
 
 static void
@@ -1776,6 +1809,7 @@ gst_vaapisink_init (GstVaapiSink * sink)
   sink->rotation = DEFAULT_ROTATION;
   sink->rotation_req = DEFAULT_ROTATION;
   sink->keep_aspect = TRUE;
+  sink->signal_handoffs = DEFAULT_SIGNAL_HANDOFFS;
   gst_video_info_init (&sink->video_info);
 
   for (i = 0; i < G_N_ELEMENTS (sink->cb_values); i++)
