@@ -20,7 +20,8 @@
  *  Boston, MA 02110-1301 USA
  */
 
-#include "gst/vaapi/sysdeps.h"
+#include "gstcompat.h"
+#include <unistd.h>
 #include <gst/vaapi/gstvaapisurface_drm.h>
 #include <gst/vaapi/gstvaapisurfacepool.h>
 #include <gst/vaapi/gstvaapiimagepool.h>
@@ -41,8 +42,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapivideomemory);
 /* --- GstVaapiVideoMemory                                              --- */
 /* ------------------------------------------------------------------------ */
 
-static void
-gst_vaapi_video_memory_reset_image (GstVaapiVideoMemory * mem);
+static void gst_vaapi_video_memory_reset_image (GstVaapiVideoMemory * mem);
 
 static guchar *
 get_image_data (GstVaapiImage * image)
@@ -370,7 +370,7 @@ gst_vaapi_video_memory_reset_surface (GstVaapiVideoMemory * mem)
 gboolean
 gst_vaapi_video_memory_sync (GstVaapiVideoMemory * mem)
 {
-  g_return_val_if_fail (mem, NULL);
+  g_return_val_if_fail (mem, FALSE);
 
   return ensure_surface_is_current (mem);
 }
@@ -773,6 +773,7 @@ gst_vaapi_dmabuf_memory_new (GstAllocator * allocator, GstVaapiVideoMeta * meta)
   GstVaapiSurface *surface;
   GstVaapiSurfaceProxy *proxy;
   GstVaapiBufferProxy *dmabuf_proxy;
+  gint dmabuf_fd;
   const GstVideoInfo *vip;
   guint flags;
 
@@ -803,8 +804,11 @@ gst_vaapi_dmabuf_memory_new (GstAllocator * allocator, GstVaapiVideoMeta * meta)
   gst_vaapi_video_meta_set_surface_proxy (meta, proxy);
   gst_vaapi_surface_proxy_unref (proxy);
 
-  mem = gst_dmabuf_allocator_alloc (allocator,
-      gst_vaapi_buffer_proxy_get_handle (dmabuf_proxy),
+  dmabuf_fd = gst_vaapi_buffer_proxy_get_handle (dmabuf_proxy);
+  if (dmabuf_fd < 0 || (dmabuf_fd = dup (dmabuf_fd)) < 0)
+    goto error_create_dmabuf_handle;
+
+  mem = gst_dmabuf_allocator_alloc (allocator, dmabuf_fd,
       gst_vaapi_buffer_proxy_get_size (dmabuf_proxy));
   if (!mem)
     goto error_create_dmabuf_memory;
@@ -834,6 +838,12 @@ error_create_dmabuf_proxy:
     gst_vaapi_surface_proxy_unref (proxy);
     return NULL;
   }
+error_create_dmabuf_handle:
+  {
+    GST_ERROR ("failed to duplicate DMABUF handle");
+    gst_vaapi_buffer_proxy_unref (dmabuf_proxy);
+    return NULL;
+  }
 error_create_dmabuf_memory:
   {
     GST_ERROR ("failed to create DMABUF memory");
